;=======================================;
;Title: DBZ III: Killer Androids 	;
;Huffman Decoder	    		;
;=======================================;

;$f6-$ff

.org $fa10
.mem 8
.index 8

decode:
	lda $f8			;In order for the variable control codes to work properly
	sta $fb			;we need to preserve the bit counter...

	lda $f9			;The WIP character being shifted...
	sta $fc

	lda $60			;And the load index.
	sta $fd

table_set:
	lda #$59		;Look-up table starts at $3fa69 in .NES file.
	sta $f6

	lda #$fa
	sta $f7
	ldx $f8

check:
	ldy #$00		;Start at the top of the table.
	lda ($f6),y
	beq leaf_found		;If the first byte is #$00, then the next
				;byte is the decoded value.

	dex			;Decrement X (count) to determine how
	bpl bit_shift		;how many bits left in current byte.
				;Sign bit will always be cleared (plus) unless count was #$00 before
				;the preceding dex.

get_byte:
	ldx #$7			;7 bits per byte (starting at 0).

	ldy $60
	lda ($56),y
	sta $f9

	inc $60			;Increment index so the right byte is read next time.

	bne reset_y		;Manually manipulate the pointer if the index exceeds #$FF.
	inc $57

reset_y:
	ldy #$00		;Load Y with #$00 so we don't use $60 in the node pointer arthimetic ahead.

bit_shift:
	asl $f9			;Shift the next bit to be tested into the Carry flag.
	bcc next_level		;If C equals 0, follow the left branch; if C equals 1, follow the right.

	iny			;Access the right branch.

next_level:
	lda ($f6),y		;The relative address to the next node.

	clc			;Add the relative address of the next node
	adc $f6			;to the current node pointer.
	sta $f6
	bcc check

	inc $f7
	bcs check

leaf_found:
	iny
	lda ($f6),y		;The decoded value
	stx $f8
	sta $fa			;Store the decoded value in RAM so we don't have to disturb the "stock" operations
				;too much.
	rts			;Run the decoded value through the normal
				;text routine