;=======================================;
;Title: DBZ III: Killer Androids 	;
;Control Codes - Huffman Style 		;
;=======================================;

.org $da17
.mem 8
.index 8

text_start:
	lda $fe			;Very hacky way of handling the
	cmp #$e0		;dictionary encoded text.
	bcs ctrl_check

	jsr read_byte

	cmp #$80		;New control code to replace the $EDXX code.
	bcc text		;Now, $8X will be functionally the same as the
	clc			;aforementioned control code, except it'll only
	cmp #$90		;require one byte instead of two, thus increasing
	bcs ctrl_check		;the efficacy of the compression.


	jmp ctrl_space

ctrl_check:
	clc
	cmp #$e0
	bcc text
	sbc #$e0
	jsr $cb7d

control_ptrs:
	.db $8a, $da		;e0_code
	.db $9e, $da		;e1_code
	.db $a3, $da		;e2_code
	.db $a8, $da		;e3_code
	.db $b7, $da		;e4_code
	.db $bb, $da		;e5_code
	.db $bf, $da		;e6_code
	.db $cc, $da		;e7_code
	.db $e3, $da		;e8_code
	.db $0f, $db		;e9_code
	.db $2d, $db		;ea_code
	.db $41, $db		;eb_code
	.db $4f, $db		;ec_code
	.db $5d, $db		;ed_code
	.db $77, $db		;ee_code
	.db $aa, $db		;ef_code
	.db $af, $db		;f0_code
	.db $c8, $da		;f1_code
	.db $d9, $da		;f2_code

text:
	ldx $61			;Load the number of characters left that can be displayed on
	bne ram_write		;the current line. If it's zero, we need to insert a line break
	jmp ee_code		;so the text doesn't spill over the window borders.

ram_write:
	ldx $0578		;The value used to set-up the X register for reading/writing consecutive
				;bytes of data to vram.

	stx $f5			;The current X value will be used to make sure the vram writing won't go
				;crazy during the Help menu.

	jsr char_decode		;Resolve the dakuten/handakuten characters into a "base" and a "subscript".
	sta $057e,x		;The "base" will be in the accumulator when the routine returns.

x_update:
	lda #$02		;Update the X register...in a very retarded way.
	jsr $ccbd

vram_addy:
	lda $54			;Lo byte of the vram address to start writing at.
	sta $057b,x
	inc $54
	lda $55			;Hi byte.
	sta $057c,x

vram_branch_set:
	lda #$01		;Used to branch the vram routine to the
	sta $ff			;text specific code.

prep_next:
	dec $61			;Decrement the number of characters left that can be displayed on current line.
	dec $5d			;Decrement the number of characters left to read before writing
	bne text_start		;to vram.
	rts

e0_code:
	lda $64			;Reset the number of characters left on current line.
	sta $62

	lda $52			;Reset the vram address for the new window.
	sta $54
	lda $53
	sta $55

	jsr joy_pad		;Alter other data that is presumably related to the window.
	
	lda #$40
	jmp end_control

e1_code:
	lda #$01
	jmp end_control

e2_code:
	lda #$02
	jmp end_control

e3_code:
	lda #$04

timed_wait:
	jsr end_control

	jsr read_byte			;The byte following #$E3 determines how
	sta $5f				;long the wait is. #$01 is roughly equal to
	lda #$00			;one second in the real world.
	sta $5e
	rts

e4_code:
	lda #$08
	bne timed_wait

e5_code:
	lda #$10
	bne timed_wait

e6_code:
	jsr read_byte
	jsr $c40f
	jmp text_start

f1_code:
	lda #$40
	bne mid_e7

e7_code:
	lda #$00
	sta $66
	lda #$80

mid_e7:
	eor $65
	sta $65
	jmp text_start

f2_code:
	lda #$f2			;Set a byte in memory so the dictionary encoded
	sta $fe				;text is compatible with the huffman routine.
					;This is reset to zero when the terminating byte
					;for the entry is encountered below.

	jsr read_byte			;The next byte is the entry number.
	jmp dict_decode

e8_code:
	lda #$e8			;See above explanation at f2_code.
	sta $fe

	jsr read_byte			;The next byte is used to determine which
	tay				;dictionary entry corresponds to the character.
	lda $060b,y			;(Chemist option in the first town)

dict_decode:
	jsr $dc82			;Loads the pointer for the dictionary entry.
	ldy $060a			;Entry index.

	lda ($a5),y			;Read a byte of the dictionary entry.
	beq dict_end			;If zero, we're done so we reset the index.

	inc $060a			;Increment index for the next read.
	ldy $60				;Reload the text index.
	bne dec_index			;If index != 0, decrement it.

	dec $57				;Else, decrement both the pointer...

dec_index:
	jsr huff_fix			;...and the index.
	jmp text			;Treat the decoded byte like any other.

dict_end:
	sta $060a			;Reset entry index.
	sta $fe				;Reset the dictionary check byte. 

	jmp text_start

e9_code:
	ldy $060a			;Load index.
	cpy #$06			;If Y >= #$06, we're done.
	bcs null_60a

	lda $060e,y			;Else, load the number data.
	bmi null_60a			;If negative (#$FF indicates the end)
					;perform a little arithmetic.

addition:
	clc
	adc #$04
	inc $060a			;Increment the load index.

	ldy $60				;Reload text index.
	bne dec_index_two		;If the text index is zero...

	dec $57				;...decrement the pointer...

dec_index_two:
	jsr huff_fix			;...and the index.
	jmp text			;Treat the byte from ram like any other
					;character.

ea_code:
	ldy $060a			;see e9_code
	cpy #$06
	bcs null_60a

	lda $0614,y
	bpl addition

null_60a:
	lda #$00
	sta $060a
	jmp text_start

eb_code:
	ldy $060a			;see e9_code
	cpy #$06
	bcs null_60a

	lda $061a,y
	bpl addition
	bmi null_60a

ec_code:
	jsr read_byte
	sta $67

	lda #$00
	sta $68

	lda #$20
	jmp end_control

ed_code:
	jsr read_byte			;The next byte determines how many spaces
	sta $00				;we want.

add_space:
	lda $61				;Subtract this value from the char counter
	sec				;and store it back at $61.
	sbc $00
	bcs mid_ed

	lda #$00

mid_ed:
	sta $61

	lda $54				;Add the value we got to the vram address
	clc				;and store it back in $54. That way, we
	adc $00				;change where the next character will be
	sta $54				;displayed.
	jmp text_start			;Since there's nothing to display, we can
					;safely read another character.

ee_code:
	inc $62				;Increment the current line number.
	lda $62				;Make sure it doesn't exceed 12 lines max.
	cmp $64
	bcs win_reset			;If it does, reset window data.

line_break:
	lda $52				;Get rid of the leading digit
	and #$1f			;so that $00 will now equal between
	sta $00				;#$00 and #$0F.

	lda $54				;Very interesting way of setting up the
	and #$e0			;vram for a new line.
	ora $00
	adc #$20			;If this results in a carry
	sta $54				;we need to increment the hi byte of the
	bcc reset_char			;vram address to reflect this.

	inc $55

reset_char:
	lda $63				;Reset the charcter counter.
	sta $61
	jmp branch_2			;Reset $5C-$5D.

win_reset:
	lda $52				;Reset the vram address.
	sta $54
	lda $53
	sta $55
	jsr branch_2

	lda #$41
	jmp end_control

ef_code:
	lda #$00
	sta $51
	rts

f0_code:
	jsr read_byte			;Changes the pointer mid-string.
	ldy #$00
	sty $60
	jmp $c44b

f0_mid:
	lda $5e
	ora $5f
	beq f0_end

	php
	dec $5e
	bpl f0_pull

	dec $5f
	lda #$3b
	sta $5e

f0_pull:
	plp

f0_end:
	rts

dbae:
	and $51
	sta $51
	rts

end_control:
	ora $51
	sta $51
	rts

read_byte:
	jsr $fa10			;Decompression time!

read_end:
	lda $fa				;Reload the byte into the accumulator.
	rts

joy_pad:
	lda $3c				;$3C is directly related to the Joy Pad,
	and #$01			;as it's status changes depending upon
	bne branch_2			;which button is pressed.
					;If $3C = #$00, go to branch_2.

	lda $5a				;The rest of this, from what I've been
	sec				;able to deduce, is a very weird way
	sbc #$0a			;of testing things.
	sta $5a

	lda $5b				;I can say that $58-$59 are constants, while
	sbc #$00			;$5A-$5B seem to be "work copies" of
	sta $5b				;$58-$59 respectively.
	bcc branch_1

	lda $5a
	sec
	sbc $58

	lda $5b
	sbc $59
	bcs branch_2

branch_1:
	lda $58
	sta $5a

	lda $59
	sta $5b

branch_2:
	lda #$00			;In the end, the characters-left-before-vram-write
	sta $5c				;variable and another variable are reset.
	sta $5d
	rts

char_decode:
	pha
	lda #$01
	sta $057d,x
	pla
	rts

huff_fix:
	pha				;Preserve the accumulator.
					;This is really only necessary
					;for the dictionary code.
					;We don't want the byte to be
					;destroyed.

	lda $fb				;bit_counter
	sta $f8

	lda $fc				;WIP byte
	sta $f9

	lda $fd				;index
	sta $60

	pla

	rts

ctrl_space:
	cmp #$80
	beq ctrl_line

	and #$0f			;Low nibble determines how many spaces.
	sta $00
	jmp add_space			;Use the original routine.

ctrl_line:
	lda $0643			;Checks to see if a there's a Save in
	bne ctrl_line_end		;Data 1. If not, add a line break.
	jmp ee_code

ctrl_line_end:
	lda #$00			;This will trigger the word wrap.
	sta $61

	lda #$01			;This is a "dead read", meaning
					;this space (#$01 = space) will be
					;ignored and not displayed.
	jmp text