;=======================================;
;Title: DBZ III: Killer Androids 	;
;Nametable load routine rewrite	    	;
;=======================================;
;The purpose of this code is to get rid ;
;of the annoying dakuten/handakuten line;
;once and for all. Previous attempts	;
;were made using less extreme methods of;
;removal, but all failed.		;
;This *should* deal with this problem	;
;once and for all			;
;=======================================;
;Original code can be found starting at ;
;$CC4B in the last bank of PRG-ROM.	;
;=======================================;
;New code can be found starting at $FA06;
;=======================================;

;=======================================;
;6 bytes of RAM are used when dealing	;
;with text. They are as follows.	;
;=======================================;
;$579,x - The number of VRAM writes to	;
;	  make. In the case of text,	;
;	  this is going to always be 2.	;
;					;
;$57a,x - Mirroring Select information.	;
;					;
;$57b,x - Low byte of the VRAM address	;
;	  to write to.			;
;					;
;$57c,x - High byte of the VRAM address	;
;	  to write to.			;
;					;
;$57d,x - The dakuten/handakuten data.	;
;					;
;$57e,x - The 'base' character data.	;
;					;
;$f5	- Used to properly handle the	;
;	  text writing.			;
;=======================================;



.org $ff10
.mem 8
.index 8

code_start:
	lda $2002		;Vblank.
	ldx #$00		;Load X with 0 because we want to start at the beginning of
				;the data to be transfered into VRAM.

	beq check_x

load_y:
	ldy $0579,x		;Load Y with the number of vram writes to make.

ppu_set_up:
	lda $057a,x
	sta $8009		;Set up the Mirroring/Page Select.

	and #$04
	sta $2000		;Set PPU to Auto-Increment by 32.

vram_set_up:
	lda $057c,x		;Set up the PPU with the address in VRAM we want to
	sta $2006		;write to.
	lda $057b,x
	sta $2006

test_text:
	lda $ff
	cmp #$01
	bne pre_vram

	cpx $f5
	bne ppu_text_prep

test_reset:
	lda #$00
	sta $ff
	sta $f5

ppu_text_prep:
	dey			;Decrement Y to compensate for only one VRAM write.
	inx			;Increment X so we retrieve data from $057E,x (the base
				;character) to display INSTEAD of $057D,x (the dakuten/
				;handakuten character).

pre_vram:
	tya			;Transfer the value in Y.
	lsr			;Shift it right.
	bcs vram_access_2	;If the least-significant bit is 1
				;only write to VRAM once this run through (the routine).
				;If the least-significant bit is 0
				;write to VRAM twice this run through.

vram_access_1:
	lda $057d,x		;This will read in the dakuten/handakuten information from RAM,
	sta $2007		;and load it into the nametable.
	inx			;Increment X so the base character will be read next.
	dey

vram_access_2:
	lda $057d,x		;This loads the base character into the nametable on the line
	sta $2007		;below the previously loaded dakuten/handakuten.
	inx
	dey
	bne vram_access_1	;if (y != 0) {goto vram_access_1;}
				;This won't happen when dealing with displaying text.

preamble_to_the_end:
	inx			;X is incremented 4 times, so that we read in $0579,(x+4),
	inx			;which will be the termination byte of the current character.
	inx			;If it isn't 00, then we still have characters left to display.
	inx

check_x:	
	cpx $0578
	bcc load_y

end_of_routine:
	lda #$00		;Fail-safe so this routine isn't unintentionally executed again.
	sta $0578
	sta $ff
	rts